@@ -5,9 +5,11 @@ from pathlib import Path
 
 import gspread
 
-from contendo_utils import ProUtils
-from contendo_utils import BigqueryUtils
-from contendo_utils import ContendoConfigurationManager
+#from contendo_utils import ProUtils
+#from contendo_utils import BigqueryUtils
+#from contendo_utils import ContendoConfigurationManager
+from contendo_utils import *
+import logging
 
 NFL_DOMAIN_NAME = 'Football.NFL'
 NFL_DOMAIN_CONFIG_GID = 274419028
@@ -34,12 +36,15 @@ class NFLStatsQueries():
         # data file name
         self.resourceDir = 'resource'
         self.pbpDataFileName = '{}/pbp_data.csv'.format(self.resourceDir)
+        self.playersDataFileName = '{}/players_data.csv'.format(self.resourceDir)
         self.pbpDF = None
         self.gamesDF = None
+        self.playersDF = None
 
+    @contendo_classfunction_logger
     def _generate_pbp_query(self):
         exceptionList = [
-            'description', 'blockingPlayer', 'penalty', 'recoveringTeam',
+            'description', 'penalizedPlayer', 'blockingPlayer', 'penalty', 'recoveringTeam', 'fumblingTeam',
             'homePlayersOnField', 'awayPlayersOnField', 'penalties', 'subPlays', 'teamReferences',
             'interceptedAtPosition', 'passedFromPosition',
         ]
@@ -60,7 +65,7 @@ class NFLStatsQueries():
                 fieldname = calc_fieldname(name, parent)
                 if field['type'] == 'RECORD':
                     if field['mode'] == 'REPEATED':
-                        print(name)
+                        logger.debug(name)
                         continue
                     else:
                         aggregate_fieldslist(field['fields'], fieldname)
@@ -81,6 +86,7 @@ class NFLStatsQueries():
             os.mkdir(self.resourceDir)
         _pbpDF.to_csv(self.pbpDataFileName)
 
+
     def _read_pbp_data(self):
         #
         # Read the pbp data from file.
@@ -91,6 +97,41 @@ class NFLStatsQueries():
             self.pbpDF = pd.read_csv(self.pbpDataFileName)
             self.pbpDF['all']=1
             self.pbpDF['count']=1
+            self.pbpDF['counter']=1
+        self.pbpDF['yardsRushed'] = pd.to_numeric(self.pbpDF['yardsRushed'])
+
+    @contendo_classfunction_logger
+    def update_players_data(self):
+        playersQuery = ProUtils.get_string_from_file('{}/queries/players_info.sql'.format(Path(__file__).parent))
+        _playersDF = self.bqu.execute_query_to_df(playersQuery)
+        logger.debug('Read players query data, shape=%s, columns=%s', _playersDF.shape, _playersDF.columns)
+        ProUtils.create_path_directories(self.resourceDir)
+        _playersDF.to_csv(self.playersDataFileName)
+        logger.debug('Updated players file %s', self.playersDataFileName)
+
+    @contendo_classfunction_logger
+    def _read_players_data(self):
+        #
+        # Read the pbp data from file.
+        logger.debug('Checking players file existance')
+        if not os.path.exists(self.playersDataFileName):
+            self.update_players_data()
+
+        logger.debug('Checking players DF existance')
+        if self.playersDF is None:
+            self.playersDF = pd.read_csv(self.playersDataFileName)
+
+    @contendo_classfunction_logger
+    def _get_player_by_id(self, playerId):
+        self._read_players_data()
+        playerDF = self.playersDF[self.playersDF.playerId == playerId]
+        if playerDF.shape[0] == 0:
+            # Error
+            logger.error('Player id %s not found', playerId)
+            return {}
+
+        return dict(playerDF.iloc[0])
+
 
     def get_games(self, filter='game.homeTeam!=""'):
         if self.gamesDF == None:
@@ -141,10 +182,11 @@ class NFLStatsQueries():
             elif _function == 'count':
                 return '{}'.format(condition)
             else:
-                return '({}) & (df.{}!="")'.format(condition, statDef['AggField'])
+                return '({})'.format(condition, statDef['AggField'])
 
         return 'True'
 
+    @contendo_classfunction_logger
     def _save_trace_df(self, traceDF, initialColumns, spreadId=None, sheetName=None):
         if not spreadId:
             spreadId = '1Q5O3ejSyEDZrlFXX04bOIWOqMxfiHJYimunZKtpFswU'
@@ -168,26 +210,25 @@ class NFLStatsQueries():
             gc = gspread.Client(credentials)
             spread = Spread(spreadId, client=gc)
             spread.df_to_sheet(traceDF[finalColumns], index=False, sheet=sheetName, start='A1', replace=True)
-            print('trace can be found in this url:', spread.url)
+            logger.info('trace can be found in this url: %s', spread.url)
         else:
             fileName = '{}.csv'.format(sheetName)
             traceDF[finalColumns].to_csv(fileName)
-            print ('Trace to file ', fileName)
+            logger.info ('Trace to file %s', fileName)
             try:
                 import google.colab
                 IN_COLAB = True
                 from google.colab import files
-                print ('Downloading', fileName)
+                logger.debug ('Downloading %s', fileName)
                 files.download(fileName)
             except Exception as e:
-                print ('Error getting trace file {}, {}'.format(fileName, e))
+                logger.warning('Error getting trace file %s', fileName)
                 IN_COLAB = False
 
-    def pbp_get_stat(self, statName, object, playDimention='all', gameDimention='game', aggfunc=None, playType=None, filter='True', trace=False):
+    @contendo_classfunction_logger
+    def pbp_get_stat(self, statName, object, playDimention='all', gameDimention='game', aggfunc=None, playType=None, filter='True', trace=False, playerId = None, teamId=None):
         assert statName in self.statsDict, "Illegal statName: '{}', must be one of {}".format(statName, self.statsDict.keys())
         assert object in self.ptmapDict, "Illegal object: '{}', must be one of {}".format(object, self.ptmapDict.keys())
-        #assert playType in self.statsDict, "Illegal playType: '{}', must be one of {}".format(statName, self.statsDict.keys())
-        #assert aggfunc in self.statsDict, "Illegal aggfunc: '{}', must be one of {}".format(statName, self.statsDict.keys())
         assert playDimention in self.playDimentionsDict, "Illegal statName: '{}', must be one of {}".format(playDimention, self.playDimentionsDict.keys())
         assert gameDimention in self.gameDimentionsDict, "Illegal gameDimention: '{}', must be one of {}".format(gameDimention, self.gameDimentionsDict.keys())
         #
@@ -203,19 +244,24 @@ class NFLStatsQueries():
             'filtercond': filter,
             'isInplayCond': "(df.playType=='penalty') | (df.isNoPlay!=True)"
         }
+        aggField = statDef['AggField']
         df = self.pbpDF
+        logger.debug('before aggfield processing %s', aggField)
+        df[aggField] = pd.to_numeric(df[aggField])
+        #queryeval = 'df[({statcond}) & ({gamecond}) & ({playcond}) & ({filtercond}) & ({isInplayCond})]'.format(**queryInst)
         queryeval = 'df[({statcond}) & ({gamecond}) & ({playcond}) & ({filtercond}) & ({isInplayCond})]'.format(**queryInst)
         try:
+            logger.debug('before filtering: %s', queryeval)
             filteredDF = eval(queryeval)
-            #print(filteredDF.shape, df.shape)
+            logger.debug('Filtered DF shape: %s, Main DF shape: %s', filteredDF.shape, df.shape)
         except Exception as e:
-            print("Error evaluating filtering statemet: {}, error: {}".format(queryeval, e))
+            logger.exception("Error evaluating filtering statemet: %s", queryeval)
             raise e
 
         #
         # return empty if no answers
         if filteredDF.shape[0]==0:
-            print ('ZERO results for filter {}, total plays {}', queryeval, df.shape[0])
+            logger.warning ('ZERO results for filter %s, total plays %s', queryeval, df.shape[0])
             return pd.DataFrame()
 
         objectDef = self.ptmapDict[object]
@@ -223,29 +269,68 @@ class NFLStatsQueries():
         if statObject=='team':
             groupby = "['{object}_id', '{object}_name']"
         else:
-            groupby = "['{object}_id', '{object}_firstName', '{object}_lastName', '{object}_position', '{team}_name']"
+            groupby = "['{object}_id', '{object}_firstName', '{object}_lastName', '{object}_position', '{team}_id', '{team}_name']"
         groupby = groupby.format(object=object, team=objectDef['TeamType'])
 
-        aggField = statDef['AggField']
-        groupingeval = "filteredDF.groupby({groupby}, as_index=False).agg({}'{aggField}': '{aggFunc}', 'count': 'count'{}).sort_values(by='{aggField}', ascending=False)"
+        groupingeval = "filteredDF.groupby({groupby}, as_index=False).agg({}'{aggField}': '{aggFunc}', 'count': 'count', 'gameid': pd.Series.nunique {}).sort_values(by='{aggField}', ascending=False)"
 
         if not aggfunc:
             aggfunc = statDef['Function']
+        logger.debug('before grouping')
         groupingeval = groupingeval.format('{', '}',groupby=groupby, aggField=aggField, aggFunc=aggfunc)
 
         try:
             finalDF = eval(groupingeval)
         except Exception as e:
-            print("Error evaluating aggregation statemet: {}, error: {}".format(groupingeval, e))
+            logger.exception("Error evaluating aggregation statemet: %s", groupingeval)
             raise e
 
         if trace:
             self._save_trace_df(filteredDF, finalDF.columns, sheetName='trace-{}-{}-{}-{}'.format(statName, object, playDimention, gameDimention))
+
+        if statObject=='team':
+            finalDF.columns=['teamId', 'teamName', 'statValue', 'count', 'nGames']
+        else:
+            finalDF.columns=['playerId', 'firstName', 'lastName', 'position', 'teamId', 'teamName', 'statValue', 'count', 'nGames']
+
         #
         # add the rank - denserank.
-        finalDF['rank'] = finalDF[aggField].rank(method='dense', ascending=False)
+        finalDF['rank'] = finalDF['statValue'].rank(method='dense', ascending=False)
+        cols = finalDF.columns.copy()
+        if teamId:
+            finalDF = finalDF[finalDF.teamId==teamId]
+            if finalDF.shape[0] == 0:
+                record=dict()
+                for col in finalDF.columns:
+                    record[col] = None
+                record['teamId'] = teamId
+                record['statValue'] = 0
+                record['count'] = 0
+                record['rank'] = -1
+                finalDF = pd.DataFrame([record])
+        if playerId:
+            finalDF = finalDF[finalDF.playerId==playerId]
+            if finalDF.shape[0] == 0:
+                playerDict = self._get_player_by_id(playerId)
+                record = dict()
+                for col in finalDF.columns:
+                    record[col] = None
+                record['playerId'] = playerId
+                record['firstName'] = playerDict.get('firstName', '')
+                record['lastName'] = playerDict.get('lastName', '')
+                record['teamId'] = playerDict.get('teamId', 0)
+                record['statValue'] = 0
+                record['count'] = 0
+                record['rank'] = -1
+                finalDF = pd.DataFrame([record])
+        finalDF=finalDF[cols]
+        finalDF['statName'] = statName
+        finalDF['statObject'] = object
+        finalDF['playDimention'] = playDimention
+        finalDF['gameDimention'] = gameDimention
         return finalDF
 
+    @contendo_classfunction_logger
     def pbp_get_composed_stat(self, compstat, object, playDimention='all', gameDimention='game', filter='True'):
         assert compstat in self.compStatsDict, "Illegal statName: '{}', must be one of {}".format(statName, self.compStatsDict.keys())
         compstatDef = self.compStatsDict[compstat]
@@ -257,9 +342,9 @@ class NFLStatsQueries():
         # define the index-key(s) for team/player
         objectDef = self.ptmapDict[object]
         if objectDef['StatObject'] == 'team':
-            key = object+'_id'
+            key = 'teamId'
         else:
-            key = [object+'_id', objectDef['TeamType']+'_name']
+            key = ['playerId', 'teamId']
         #
         # get the numerator data
         numeratorDF = self.pbp_get_stat(numerator, object, playDimention, gameDimention, filter=filter).set_index(key)
@@ -275,22 +360,25 @@ class NFLStatsQueries():
             denominatorDF,
             rsuffix='_dn',
             on=key,
-            how='left'
+            how='left',
+
         )
-        #print(df)
-        #print(df[['offenseTeam_name',  'count', denominatorDef['AggField']+'_dn']])
-        #numeratorDF[denominatorDef['AggField']+'_dn'] = df[denominatorDef['AggField']+'_dn']
-        df[compstat] = df[numeratorDef['AggField']]/df[denominatorDef['AggField']+'_dn']*compstatDef['StatRatio']
-        df.sort_values(by=compstat, ascending=False, inplace=True)
-        df['rank'] = df[compstat].rank(method='dense', ascending=False)
+        df['statValue'] = df['statValue']/df['statValue_dn']*compstatDef['StatRatio']
+        df.sort_values(by='statValue', ascending=False, inplace=True)
+        df['rank'] = df['statValue'].rank(method='dense', ascending=False)
         retCols = list()
         for col in df.columns:
             if col.find('_dn')==-1 or col==denominatorDef['AggField']+'_dn':
                 retCols.append(col)
+        df['statName'] = compstat
+        df['statObject'] = object
+        df['playDimention'] = playDimention
+        df['gameDimention'] = gameDimention
+        df = df.reset_index(level = ['playerId', 'teamId'])
         return df[retCols]
 
-
-def test_all_stats(generator, startTime=dt.now()):
+@contendo_function_logger
+def test_all_stats(generator):
     counter=dict()
     results=[]
     for statName, statDef in generator.statsDict.items():
@@ -305,8 +393,7 @@ def test_all_stats(generator, startTime=dt.now()):
                         continue
 
                     if gameCondDef['Condition']=='' or playCondDef['Condition']=='':
-                        #print ('skipping', statDef, gameCondDef, playCondDef)
-                        #return
+                        logger.debug ('skipping %s, %s, %s', statDef, gameCondDef, playCondDef)
                         continue
                     if statDef['playType'] != 'all' and playCondDef['playType'].find(statDef['playType'])==-1:
                         continue
@@ -314,7 +401,7 @@ def test_all_stats(generator, startTime=dt.now()):
                     df = generator.pbp_get_stat(statName, object, playCondCode, gameCondCode)
                     isResults = (df.shape[0]>0)
                     counter[isResults] = counter.get(isResults, 0)+1
-                    print (counter[isResults], isResults, df.shape, statName, object, playCondCode, gameCondCode, dt.now() - startTime)
+                    logger.debug ('%s, %s, %s, %s, %s, %s, %s', counter[isResults], isResults, df.shape, statName, object, playCondCode, gameCondCode)
                     if isResults:
                         results.append(
                             {
@@ -334,9 +421,10 @@ def test_all_stats(generator, startTime=dt.now()):
     spread = Spread(generator.ccm.get_domain_docid('Football.NFL'))
     spread.df_to_sheet(resultsDF, index=False, sheet='PBP Stats results', start='A1', replace=True)
 
+@contendo_function_logger
 def test():
     startTime=dt.now()
-    pd.set_option('display.max_columns', 12)
+    pd.set_option('display.max_columns', 20)
     pd.set_option('display.width', 200)
     #os.environ['CONTENDO_AT_HOME'] = 'y'
     os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = "{}/sportsight-tests.json".format(os.environ["HOME"])
@@ -344,16 +432,19 @@ def test():
     os.chdir('{}/tmp/'.format(os.environ["HOME"]))
     generator = NFLStatsQueries()
     #print(generator._generate_pbp_query())
-    print('Start updating pbp data', dt.now() - startTime)
+    logger.info('Start updating pbp data, delta = %s', dt.now() - startTime)
     #generator.update_pbp_data()
-    print('Start querying data', dt.now() - startTime)
-    df = generator.pbp_get_stat('sacks', 'soloTacklingPlayer', 'all', 'game',aggfunc='count', filter="df.season=='2019-regular'", trace=True)
-    #df = generator.pbp_get_composed_stat('yardsPerCatch', 'passingPlayer')
+    logger.info('Start querying data, delta = %s', dt.now() - startTime)
+    #df = generator.pbp_get_stat(statName='rushes', object='rushingPlayer', playDimention='conversion3', gameDimention='game', filter="df.season=='2019-regular'")
+    #logger.info('Columns={}, Shape={}\n\n{}\n\n'.format(df.columns, df.shape, df))
+    df = generator.pbp_get_stat('rushes', 'rushingPlayer', 'conversion3', '4thQ', filter="df.season=='2019-regular'", trace=False, playerId=12606)
+    logger.info('Columns={}, Shape={}\n{}'.format(df.columns, df.shape, df))
+    #df = generator.pbp_get_composed_stat('yardsPerCarry', 'rushingPlayer')
+    #logger.info('Columns={}, Shape={}\n{}'.format(df.columns, df.shape, df))
     #df = generator.get_games()
-    print(df.columns, df.shape,'\n', df)
-    #print ('shape: {}\nIndex: {}\nColumns: {}\n'.format(df.shape, df.index, df.columns))
-    #test_all_stats(generator)
-    print('Done', dt.now() - startTime)
+    test_all_stats(generator)
+    logger.info('Done, delta=%s', dt.now() - startTime)
 
 if __name__ == '__main__':
+    contendo_logging_setup(default_level=logging.DEBUG)
     test()
